@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -37,12 +39,12 @@
 #include <core/dev/net_device_table_mgr.h>
 #include "core/dev/ring_allocation_logic.h"
 #include "core/sock/fd_collection.h"
-#include "core/sock/sock-redirect.h" // calling orig_os_api.epoll()
+#include "core/sock/sock-redirect.h" // calling SYSCALL(epoll)()
 #include "core/proto/route_table_mgr.h"
 #include "timer_handler.h"
 #include "event_handler_ibverbs.h"
 #include "event_handler_rdma_cm.h"
-
+#include "core/sock/sockinfo_tcp.h"
 #include "core/util/instrumentation.h"
 
 #define MODULE_NAME "evh:"
@@ -81,20 +83,19 @@
 
 #define INITIAL_EVENTS_NUM 64
 
-event_handler_manager *g_p_event_handler_manager = NULL;
+event_handler_manager *g_p_event_handler_manager = nullptr;
 
 pthread_t g_n_internal_thread_id = 0;
 
 void *event_handler_manager::register_timer_event(int timeout_msec, timer_handler *handler,
-                                                  timer_req_type_t req_type, void *user_data,
-                                                  timers_group *group /* = NULL */)
+                                                  timer_req_type_t req_type, void *user_data)
 {
     evh_logdbg("timer handler '%p' registered %s timer for %d msec (user data: %p)", handler,
                timer_req_type_str(req_type), timeout_msec, user_data);
     BULLSEYE_EXCLUDE_BLOCK_START
     if (!handler || (req_type < 0 || req_type >= INVALID_TIMER)) {
         evh_logwarn("bad timer type (%d) or handler (%p)", req_type, handler);
-        return NULL;
+        return nullptr;
     }
     BULLSEYE_EXCLUDE_BLOCK_END
 
@@ -115,7 +116,6 @@ void *event_handler_manager::register_timer_event(int timeout_msec, timer_handle
     reg_action.type = REGISTER_TIMER;
     reg_action.info.timer.handler = handler;
     reg_action.info.timer.user_data = user_data;
-    reg_action.info.timer.group = group;
     reg_action.info.timer.node = node;
     reg_action.info.timer.timeout_msec = timeout_msec;
     reg_action.info.timer.req_type = req_type;
@@ -123,6 +123,16 @@ void *event_handler_manager::register_timer_event(int timeout_msec, timer_handle
     return node;
 }
 
+void event_handler_manager::register_socket_timer_event(sockinfo_tcp *sock_tcp)
+{
+    evh_logdbg("Registering TCP socket timer: %p", sock_tcp);
+    reg_action_t reg_action;
+    memset(&reg_action, 0, sizeof(reg_action));
+    reg_action.type = REGISTER_TCP_SOCKET_TIMER;
+    reg_action.info.timer.user_data = sock_tcp;
+    post_new_reg_action(reg_action);
+}
+
 void event_handler_manager::wakeup_timer_event(timer_handler *handler, void *node)
 {
     evh_logdbg("timer handler '%p'", handler);
@@ -138,7 +148,6 @@ void event_handler_manager::wakeup_timer_event(timer_handler *handler, void *nod
     reg_action.info.timer.handler = handler;
     reg_action.info.timer.node = node;
     post_new_reg_action(reg_action);
-    return;
 }
 
 void event_handler_manager::unregister_timer_event(timer_handler *handler, void *node)
@@ -175,6 +184,16 @@ void event_handler_manager::unregister_timers_event_and_delete(timer_handler *ha
     post_new_reg_action(reg_action);
 }
 
+void event_handler_manager::unregister_socket_timer_and_delete(sockinfo_tcp *sock_tcp)
+{
+    evh_logdbg("Unregistering TCP socket timer: %p", sock_tcp);
+    reg_action_t reg_action;
+    memset(&reg_action, 0, sizeof(reg_action));
+    reg_action.type = UNREGISTER_TCP_SOCKET_TIMER_AND_DELETE;
+    reg_action.info.timer.user_data = sock_tcp;
+    post_new_reg_action(reg_action);
+}
+
 void event_handler_manager::register_ibverbs_event(int fd, event_handler_ibverbs *handler,
                                                    void *channel, void *user_data)
 {
@@ -234,17 +253,23 @@ void event_handler_manager::register_command_event(int fd, command *cmd)
     post_new_reg_action(reg_action);
 }
 
-event_handler_manager::event_handler_manager()
+event_handler_manager::event_handler_manager(bool internal_thread_mode)
     : m_reg_action_q_lock("reg_action_q_lock")
     , m_b_sysvar_internal_thread_arm_cq_enabled(safe_mce_sys().internal_thread_arm_cq_enabled)
-    , m_n_sysvar_xlio_time_measure_num_samples(safe_mce_sys().xlio_time_measure_num_samples)
     , m_n_sysvar_timer_resolution_msec(safe_mce_sys().timer_resolution_msec)
 {
     evh_logfunc("");
 
     m_cq_epfd = 0;
+    m_epfd = -1;
+    m_event_handler_tid = 0;
 
-    m_epfd = orig_os_api.epoll_create(INITIAL_EVENTS_NUM);
+    if (!internal_thread_mode) {
+        m_b_continue_running = true;
+        return;
+    }
+
+    m_epfd = SYSCALL(epoll_create, INITIAL_EVENTS_NUM);
     BULLSEYE_EXCLUDE_BLOCK_START
     if (m_epfd == -1) {
         evh_logdbg("epoll_create failed on ibv device collection (errno=%d %m)", errno);
@@ -254,12 +279,9 @@ event_handler_manager::event_handler_manager()
     BULLSEYE_EXCLUDE_BLOCK_END
 
     m_b_continue_running = true;
-    m_event_handler_tid = 0;
 
     wakeup_set_epoll_fd(m_epfd);
     going_to_sleep();
-
-    return;
 }
 
 event_handler_manager::~event_handler_manager()
@@ -288,7 +310,7 @@ void *event_handler_thread(void *_p_tgtObject)
         tasks_file += "/tasks";
         FILE *fp = fopen(tasks_file.c_str(), "w");
         BULLSEYE_EXCLUDE_BLOCK_START
-        if (fp == NULL) {
+        if (!fp) {
             evh_logpanic("Failed to open %s for writing", tasks_file.c_str());
         }
         if (fprintf(fp, "%d", gettid()) <= 0) {
@@ -311,7 +333,6 @@ void *event_handler_thread(void *_p_tgtObject)
         } else {
             evh_logdbg("Internal thread affinity not set.");
         }
-        /* cppcheck-suppress resourceLeak */
     }
 
     void *ret = p_tgtObject->thread_loop();
@@ -388,7 +409,7 @@ void event_handler_manager::stop_thread()
 
         // Wait for thread exit
         if (m_event_handler_tid) {
-            pthread_join(m_event_handler_tid, 0);
+            pthread_join(m_event_handler_tid, nullptr);
             evh_logdbg("event handler thread stopped");
         } else {
             evh_logdbg("event handler thread not running");
@@ -397,13 +418,15 @@ void event_handler_manager::stop_thread()
     m_event_handler_tid = 0;
 
     // Close main epfd and signaling socket
-    orig_os_api.close(m_epfd);
+    if (m_epfd >= 0) {
+        SYSCALL(close, m_epfd);
+    }
     m_epfd = -1;
 }
 
 void event_handler_manager::update_epfd(int fd, int operation, int events)
 {
-    epoll_event ev = {0, {0}};
+    epoll_event ev = {0, {nullptr}};
 
     if (m_epfd < 0) {
         return;
@@ -412,7 +435,7 @@ void event_handler_manager::update_epfd(int fd, int operation, int events)
     ev.events = events;
     ev.data.fd = fd;
     BULLSEYE_EXCLUDE_BLOCK_START
-    if ((orig_os_api.epoll_ctl(m_epfd, operation, fd, &ev) < 0) &&
+    if ((SYSCALL(epoll_ctl, m_epfd, operation, fd, &ev) < 0) &&
         (!(errno == ENOENT || errno == EBADF))) {
         const char *operation_str[] = {"", "ADD", "DEL", "MOD"};
         evh_logerr("epoll_ctl(%d, %s, fd=%d) failed (errno=%d %m)", m_epfd,
@@ -424,6 +447,10 @@ void event_handler_manager::update_epfd(int fd, int operation, int events)
 const char *event_handler_manager::reg_action_str(event_action_type_e reg_action_type)
 {
     switch (reg_action_type) {
+    case REGISTER_TCP_SOCKET_TIMER:
+        return "REGISTER_TCP_SOCKET_TIMER";
+    case UNREGISTER_TCP_SOCKET_TIMER_AND_DELETE:
+        return "UNREGISTER_TCP_SOCKET_TIMER_AND_DELETE";
     case REGISTER_TIMER:
         return "REGISTER_TIMER";
     case UNREGISTER_TIMER:
@@ -474,30 +501,18 @@ void event_handler_manager::post_new_reg_action(reg_action_t &reg_action)
 
 void event_handler_manager::priv_register_timer_handler(timer_reg_info_t &info)
 {
-    if (info.group) {
-        info.group->add_new_timer((timer_node_t *)info.node, info.handler, info.user_data);
-    } else {
-        m_timer.add_new_timer(info.timeout_msec, (timer_node_t *)info.node, info.handler,
-                              info.user_data, info.req_type);
-    }
+    m_timer.add_new_timer(info.timeout_msec, (timer_node_t *)info.node, info.handler,
+                          info.user_data, info.req_type);
 }
 
 void event_handler_manager::priv_wakeup_timer_handler(timer_reg_info_t &info)
 {
-    timer_node_t *node = (timer_node_t *)info.node;
-    if (node && !node->group) {
-        m_timer.wakeup_timer(node);
-    }
+    m_timer.wakeup_timer((timer_node_t *)info.node);
 }
 
 void event_handler_manager::priv_unregister_timer_handler(timer_reg_info_t &info)
 {
-    timer_node_t *node = (timer_node_t *)info.node;
-    if (node && node->group) {
-        node->group->remove_timer((timer_node_t *)info.node);
-    } else {
-        m_timer.remove_timer(node, info.handler);
-    }
+    m_timer.remove_timer((timer_node_t *)info.node, info.handler);
 }
 
 void event_handler_manager::priv_unregister_all_handler_timers(timer_reg_info_t &info)
@@ -523,11 +538,12 @@ void event_handler_manager::priv_prepare_ibverbs_async_event_queue(event_handler
     set_fd_block_mode(poll_fd.fd, false);
 
     // empty the async event queue
-    while (orig_os_api.poll(&poll_fd, 1, 0) > 0) {
+    while (SYSCALL(poll, &poll_fd, 1, 0) > 0) {
         process_ibverbs_event(i);
         cnt++;
     }
     evh_logdbg("Emptied %d Events", cnt);
+    NOT_IN_USE(cnt); // Suppress --enable-opt-log=high warning
 }
 
 void event_handler_manager::priv_register_ibverbs_events(ibverbs_reg_info_t &info)
@@ -535,15 +551,7 @@ void event_handler_manager::priv_register_ibverbs_events(ibverbs_reg_info_t &inf
     event_handler_map_t::iterator i;
     i = m_event_handler_map.find(info.fd);
     if (i == m_event_handler_map.end()) {
-        event_data_t v;
-
-        v.type = EV_IBVERBS;
-        v.ibverbs_ev.fd = info.fd;
-        v.ibverbs_ev.channel = info.channel;
-
-        /* coverity[uninit_use_in_call] */
-        /* cppcheck-suppress uninitStructMember */
-        m_event_handler_map[info.fd] = v;
+        m_event_handler_map[info.fd] = event_data_t(EV_IBVERBS, info);
         i = m_event_handler_map.find(info.fd);
 
         priv_prepare_ibverbs_async_event_queue(i);
@@ -623,16 +631,7 @@ void event_handler_manager::priv_register_rdma_cm_events(rdma_cm_reg_info_t &inf
     event_handler_map_t::iterator iter_fd = m_event_handler_map.find(info.fd);
     if (iter_fd == m_event_handler_map.end()) {
         evh_logdbg("Adding new channel (fd %d, id %p, handler %p)", info.fd, info.id, info.handler);
-        event_data_t map_value;
-
-        map_value.type = EV_RDMA_CM;
-        map_value.rdma_cm_ev.n_ref_count = 1;
-        map_value.rdma_cm_ev.map_rdma_cm_id[info.id] = info.handler;
-        map_value.rdma_cm_ev.cma_channel = info.cma_channel;
-
-        /* coverity[uninit_use_in_call] */
-        /* cppcheck-suppress uninitStructMember */
-        m_event_handler_map[info.fd] = map_value;
+        m_event_handler_map[info.fd] = event_data_t(EV_RDMA_CM, info);
 
         update_epfd(info.fd, EPOLL_CTL_ADD, EPOLLIN | EPOLLPRI);
     } else {
@@ -700,14 +699,7 @@ void event_handler_manager::priv_register_command_events(command_reg_info_t &inf
     event_handler_map_t::iterator iter_fd = m_event_handler_map.find(info.fd);
     if (iter_fd == m_event_handler_map.end()) {
         evh_logdbg("Adding new channel (fd %d)", info.fd);
-        event_data_t map_value;
-
-        map_value.type = EV_COMMAND;
-        map_value.command_ev.cmd = info.cmd;
-
-        /* coverity[uninit_use_in_call] */
-        /* cppcheck-suppress uninitStructMember */
-        m_event_handler_map[info.fd] = map_value;
+        m_event_handler_map[info.fd] = event_data_t(EV_COMMAND, info);
         update_epfd(info.fd, EPOLL_CTL_ADD, EPOLLIN | EPOLLPRI);
     }
 }
@@ -733,7 +725,17 @@ void event_handler_manager::handle_registration_action(reg_action_t &reg_action)
     }
 
     evh_logfunc("event action %d", reg_action.type);
+    sockinfo_tcp *sock;
     switch (reg_action.type) {
+    case REGISTER_TCP_SOCKET_TIMER:
+        sock = reinterpret_cast<sockinfo_tcp *>(reg_action.info.timer.user_data);
+        sock->get_tcp_timer_collection()->add_new_timer(sock);
+        break;
+    case UNREGISTER_TCP_SOCKET_TIMER_AND_DELETE:
+        sock = reinterpret_cast<sockinfo_tcp *>(reg_action.info.timer.user_data);
+        sock->get_tcp_timer_collection()->remove_timer(sock);
+        delete sock;
+        break;
     case REGISTER_TIMER:
         priv_register_timer_handler(reg_action.info.timer);
         break;
@@ -764,7 +766,7 @@ void event_handler_manager::handle_registration_action(reg_action_t &reg_action)
     case UNREGISTER_TIMERS_AND_DELETE:
         priv_unregister_all_handler_timers(reg_action.info.timer);
         delete reg_action.info.timer.handler;
-        reg_action.info.timer.handler = NULL;
+        reg_action.info.timer.handler = nullptr;
         break;
         BULLSEYE_EXCLUDE_BLOCK_START
     default:
@@ -792,7 +794,7 @@ void event_handler_manager::query_for_ibverbs_event(int async_fd)
     }
 
     // Check for ready events
-    if (orig_os_api.poll(&poll_fd, 1, 0) <= 0) {
+    if (SYSCALL(poll, &poll_fd, 1, 0) <= 0) {
         return;
     }
 
@@ -871,7 +873,7 @@ void event_handler_manager::process_rdma_cm_event(event_handler_map_t::iterator
     // Read the notification event channel
     struct rdma_event_channel *cma_channel =
         (struct rdma_event_channel *)iter_fd->second.rdma_cm_ev.cma_channel;
-    struct rdma_cm_event *p_tmp_cm_event = NULL;
+    struct rdma_cm_event *p_tmp_cm_event = nullptr;
     struct rdma_cm_event cma_event;
 
     evh_logfunc_entry("cma_channel %p (fd = %d)", cma_channel, cma_channel->fd);
@@ -904,7 +906,7 @@ void event_handler_manager::process_rdma_cm_event(event_handler_map_t::iterator
     }
 
     // Find registered event handler
-    if (cma_id != NULL) {
+    if (cma_id) {
         event_handler_rdma_cm_map_t::iterator iter_id =
             iter_fd->second.rdma_cm_ev.map_rdma_cm_id.find(cma_id);
         if (iter_id != iter_fd->second.rdma_cm_ev.map_rdma_cm_id.end()) {
@@ -950,11 +952,6 @@ void *event_handler_manager::thread_loop()
     poll_fd.events = POLLIN | POLLPRI;
     poll_fd.revents = 0;
     while (m_b_continue_running) {
-#ifdef XLIO_TIME_MEASURE
-        if (g_inst_cnt >= m_n_sysvar_xlio_time_measure_num_samples)
-            finit_instrumentation(safe_mce_sys().xlio_time_measure_filename);
-#endif
-
         // update timer and get timeout
         timeout_msec = m_timer.update_timeout();
         if (timeout_msec == 0) {
@@ -967,20 +964,24 @@ void *event_handler_manager::thread_loop()
             g_p_net_device_table_mgr) {
             m_cq_epfd = g_p_net_device_table_mgr->global_ring_epfd_get();
             if (m_cq_epfd > 0) {
-                epoll_event evt = {0, {0}};
+                epoll_event evt = {0, {nullptr}};
                 evt.events = EPOLLIN | EPOLLPRI;
                 evt.data.fd = m_cq_epfd;
-                orig_os_api.epoll_ctl(m_epfd, EPOLL_CTL_ADD, m_cq_epfd, &evt);
+                SYSCALL(epoll_ctl, m_epfd, EPOLL_CTL_ADD, m_cq_epfd, &evt);
             }
         }
 
-        uint64_t poll_sn = 0;
+        uint64_t poll_sn_rx = 0;
+        uint64_t poll_sn_tx = 0;
         if (m_b_sysvar_internal_thread_arm_cq_enabled && m_cq_epfd > 0 &&
             g_p_net_device_table_mgr) {
-            g_p_net_device_table_mgr->global_ring_poll_and_process_element(&poll_sn, NULL);
-            int ret = g_p_net_device_table_mgr->global_ring_request_notification(poll_sn);
+            g_p_net_device_table_mgr->global_ring_poll_and_process_element(&poll_sn_rx, &poll_sn_tx,
+                                                                           nullptr);
+            int ret =
+                g_p_net_device_table_mgr->global_ring_request_notification(poll_sn_rx, poll_sn_tx);
             if (ret > 0) {
-                g_p_net_device_table_mgr->global_ring_poll_and_process_element(&poll_sn, NULL);
+                g_p_net_device_table_mgr->global_ring_poll_and_process_element(
+                    &poll_sn_rx, &poll_sn_tx, nullptr);
             }
         }
 
@@ -991,20 +992,20 @@ void *event_handler_manager::thread_loop()
             }
         }
 
-        evh_logfuncall("calling orig_os_api.epoll with %d msec timeout", timeout_msec);
-        int ret = orig_os_api.epoll_wait(m_epfd, p_events, maxevents, timeout_msec);
+        evh_logfuncall("calling SYSCALL(epoll) with %d msec timeout", timeout_msec);
+        int ret = SYSCALL(epoll_wait, m_epfd, p_events, maxevents, timeout_msec);
         if (ret < 0) {
             evh_logfunc("epoll returned with error, errno=%d %m)", errno);
             continue;
         }
-        evh_logfuncall("orig_os_api.epoll found %d ready fds", ret);
+        evh_logfuncall("SYSCALL(epoll) found %d ready fds", ret);
 
         // check pipe
         for (int idx = 0; (idx < ret) && (m_b_continue_running); ++idx) {
             if (m_b_sysvar_internal_thread_arm_cq_enabled && p_events[idx].data.fd == m_cq_epfd &&
                 g_p_net_device_table_mgr) {
                 g_p_net_device_table_mgr->global_ring_wait_for_notification_and_process_element(
-                    &poll_sn, NULL);
+                    &poll_sn_rx, nullptr);
             } else if (is_wakeup_fd(p_events[idx].data.fd)) {
                 // a request for registration was sent
                 m_reg_action_q_lock.lock();
@@ -1060,7 +1061,7 @@ void *event_handler_manager::thread_loop()
             case EV_RDMA_CM:
                 int result;
                 poll_fd.fd = fd;
-                result = orig_os_api.poll(&poll_fd, 1, 0);
+                result = SYSCALL(poll, &poll_fd, 1, 0);
                 if (result == 0) {
                     evh_logdbg("error in fd %d", fd);
                     break;
@@ -1098,5 +1099,5 @@ void *event_handler_manager::thread_loop()
 
     free(p_events);
 
-    return 0;
+    return nullptr;
 }
