@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -105,7 +107,6 @@ void priv_ibv_modify_cq_moderation(struct ibv_cq *cq, uint32_t period, uint32_t
 #define FS_MASK_ON_64 (0xffffffffffffffff)
 
 #define FLOW_TAG_MASK ((1 << 20) - 1)
-int priv_ibv_query_flow_tag_supported(struct ibv_qp *qp, uint8_t port_num, sa_family_t family);
 int priv_ibv_query_burst_supported(struct ibv_qp *qp, uint8_t port_num);
 
 /* DEFINED_VERBS_VERSION:
@@ -169,13 +170,6 @@ typedef int xlio_ibv_cq_init_attr;
     ibv_create_cq(context, cqe, cq_context, channel, comp_vector)
 
 // rx hw timestamp
-#define XLIO_IBV_WC_WITH_TIMESTAMP 0
-#define xlio_wc_timestamp(wc)      0
-#define xlio_ibv_cq_init_ts_attr(attr)                                                             \
-    {                                                                                              \
-        NOT_IN_USE(attr);                                                                          \
-    }
-
 #ifdef DEFINED_IBV_CQ_TIMESTAMP
 #define XLIO_IBV_DEVICE_ATTR_HCA_CORE_CLOCK 0
 #define XLIO_IBV_VALUES_MASK_RAW_CLOCK      IBV_VALUES_MASK_RAW_CLOCK
@@ -185,14 +179,12 @@ typedef struct ibv_values_ex xlio_ts_values;
 #endif
 
 // ibv_post_send
-#define XLIO_IBV_SEND_SIGNALED IBV_SEND_SIGNALED
-#define XLIO_IBV_SEND_INLINE   IBV_SEND_INLINE
+#define XLIO_IBV_SEND_INLINE IBV_SEND_INLINE
 #ifdef DEFINED_IBV_SEND_IP_CSUM
 #define XLIO_IBV_SEND_IP_CSUM (IBV_SEND_IP_CSUM)
 #else
 #define DEFINED_SW_CSUM
 #endif
-#define xlio_ibv_send_flags         ibv_send_flags
 #define xlio_send_wr_send_flags(wr) (wr).send_flags
 #define XLIO_IBV_WR_SEND            IBV_WR_SEND
 #define xlio_ibv_wr_opcode          ibv_wr_opcode
@@ -218,41 +210,9 @@ typedef struct ibv_tso_caps xlio_ibv_tso_caps;
     (xlio_ibv_wr_opcode)(0) // Use 0 as "default" opcode when NOP is not defined.
 #endif
 
-#define xlio_ibv_post_send(qp, wr, bad_wr) ibv_post_send(qp, wr, bad_wr)
 typedef struct ibv_send_wr xlio_ibv_send_wr;
 // ibv_reg_mr
 #define XLIO_IBV_ACCESS_LOCAL_WRITE IBV_ACCESS_LOCAL_WRITE
-#ifdef DEFINED_IBV_ACCESS_ALLOCATE_MR
-#define XLIO_IBV_ACCESS_ALLOCATE_MR IBV_ACCESS_ALLOCATE_MR
-#endif
-// flow steering
-#define XLIO_IBV_FLOW_ATTR_NORMAL                IBV_FLOW_ATTR_NORMAL
-#define XLIO_IBV_FLOW_ATTR_FLAGS_ALLOW_LOOP_BACK IBV_FLOW_ATTR_FLAGS_ALLOW_LOOP_BACK
-#define XLIO_IBV_FLOW_SPEC_ETH                   IBV_FLOW_SPEC_ETH
-#define XLIO_IBV_FLOW_SPEC_IPV4                  IBV_FLOW_SPEC_IPV4
-#define XLIO_IBV_FLOW_SPEC_IPV6                  IBV_FLOW_SPEC_IPV6
-#define XLIO_IBV_FLOW_SPEC_TCP                   IBV_FLOW_SPEC_TCP
-#define XLIO_IBV_FLOW_SPEC_UDP                   IBV_FLOW_SPEC_UDP
-#define xlio_ibv_create_flow(qp, flow)           ibv_create_flow(qp, flow)
-#define xlio_ibv_destroy_flow(flow_id)           ibv_destroy_flow(flow_id)
-typedef struct ibv_flow xlio_ibv_flow;
-typedef struct ibv_flow_attr xlio_ibv_flow_attr;
-typedef struct ibv_flow_spec_ib xlio_ibv_flow_spec_ib;
-typedef struct ibv_flow_spec_eth xlio_ibv_flow_spec_eth;
-typedef struct ibv_flow_spec_ipv4 xlio_ibv_flow_spec_ipv4;
-typedef struct ibv_flow_spec_ipv6 xlio_ibv_flow_spec_ipv6;
-typedef struct ibv_flow_spec_tcp_udp xlio_ibv_flow_spec_tcp_udp;
-
-// Flow tag
-#ifdef DEFINED_IBV_FLOW_TAG
-#define XLIO_IBV_FLOW_SPEC_ACTION_TAG IBV_FLOW_SPEC_ACTION_TAG
-typedef struct ibv_flow_spec_action_tag xlio_ibv_flow_spec_action_tag;
-#define xlio_get_flow_tag(cqe) ntohl((uint32_t)(cqe->sop_drop_qpn))
-#else
-typedef struct ibv_flow_spec_action_tag_dummy {
-} xlio_ibv_flow_spec_action_tag;
-#define xlio_get_flow_tag(cqe) 0
-#endif // DEFINED_IBV_FLOW_TAG
 
 #ifdef DEFINED_IBV_CQ_ATTR_MODERATE
 typedef struct ibv_modify_cq_attr xlio_ibv_cq_attr;
@@ -354,93 +314,4 @@ typedef enum {
 
 int xlio_rdma_lib_reset();
 
-static inline void ibv_flow_spec_eth_set(xlio_ibv_flow_spec_eth *eth, uint8_t *dst_mac,
-                                         uint16_t vlan_tag, bool is_ipv4)
-{
-    eth->type = XLIO_IBV_FLOW_SPEC_ETH;
-    eth->size = sizeof(xlio_ibv_flow_spec_eth);
-    eth->val.ether_type = ntohs(is_ipv4 ? ETH_P_IP : ETH_P_IPV6);
-    eth->mask.ether_type = FS_MASK_ON_16;
-    memcpy(eth->val.dst_mac, dst_mac, ETH_ALEN);
-    memset(eth->mask.dst_mac, FS_MASK_ON_8, ETH_ALEN);
-    eth->val.vlan_tag = vlan_tag & htons(VLAN_VID_MASK);
-    eth->mask.vlan_tag =
-        eth->val.vlan_tag ? htons(VLAN_VID_MASK) : 0; // we do not support vlan options
-}
-
-template <typename T>
-static inline void ibv_flow_spec_set_single_ip(T &spec_ip_val, T &spec_ip_mask,
-                                               const ip_address &src_ip)
-{
-}
-
-typedef decltype(ibv_flow_ipv4_filter::src_ip) spec_ipv4_type;
-template <>
-inline void ibv_flow_spec_set_single_ip(spec_ipv4_type &spec_ip_val, spec_ipv4_type &spec_ip_mask,
-                                        const ip_address &in_ip)
-{
-    memcpy(&spec_ip_val, &in_ip.get_in4_addr(), sizeof(spec_ipv4_type));
-    spec_ip_mask = (!in_ip.is_anyaddr() ? FS_MASK_ON_32 : 0U);
-}
-
-typedef decltype(ibv_flow_ipv6_filter::src_ip) spec_ipv6_type;
-template <>
-inline void ibv_flow_spec_set_single_ip(spec_ipv6_type &spec_ip_val, spec_ipv6_type &spec_ip_mask,
-                                        const ip_address &in_ip)
-{
-    memcpy(&spec_ip_val, &in_ip.get_in6_addr(), sizeof(spec_ipv6_type));
-    memset(&spec_ip_mask, in_ip.is_anyaddr() ? 0 : 0xff, sizeof(spec_ipv6_type));
-}
-
-static inline void ibv_flow_spec_ip_set(xlio_ibv_flow_spec_ipv4 *ipv4, const ip_address &dst_ip,
-                                        const ip_address &src_ip)
-{
-    ipv4->type = XLIO_IBV_FLOW_SPEC_IPV4;
-    ipv4->size = sizeof(xlio_ibv_flow_spec_ipv4);
-    ibv_flow_spec_set_single_ip(ipv4->val.src_ip, ipv4->mask.src_ip, src_ip);
-    ibv_flow_spec_set_single_ip(ipv4->val.dst_ip, ipv4->mask.dst_ip, dst_ip);
-}
-
-static inline void ibv_flow_spec_ip_set(xlio_ibv_flow_spec_ipv6 *ipv6, const ip_address &dst_ip,
-                                        const ip_address &src_ip)
-{
-    ipv6->type = XLIO_IBV_FLOW_SPEC_IPV6;
-    ipv6->size = sizeof(xlio_ibv_flow_spec_ipv6);
-    ibv_flow_spec_set_single_ip(ipv6->val.src_ip, ipv6->mask.src_ip, src_ip);
-    ibv_flow_spec_set_single_ip(ipv6->val.dst_ip, ipv6->mask.dst_ip, dst_ip);
-    ipv6->val.flow_label = ipv6->mask.flow_label = 0U;
-    ipv6->val.next_hdr = ipv6->mask.next_hdr = 0U;
-    ipv6->val.traffic_class = ipv6->mask.traffic_class = 0U;
-    ipv6->val.hop_limit = ipv6->mask.hop_limit = 0U;
-}
-
-static inline void ibv_flow_spec_tcp_udp_set(xlio_ibv_flow_spec_tcp_udp *tcp_udp, bool is_tcp,
-                                             uint16_t dst_port, uint16_t src_port)
-{
-    tcp_udp->type = is_tcp ? XLIO_IBV_FLOW_SPEC_TCP : XLIO_IBV_FLOW_SPEC_UDP;
-    tcp_udp->size = sizeof(xlio_ibv_flow_spec_tcp_udp);
-    tcp_udp->val.src_port = src_port;
-    if (tcp_udp->val.src_port) {
-        tcp_udp->mask.src_port = FS_MASK_ON_16;
-    }
-    tcp_udp->val.dst_port = dst_port;
-    if (tcp_udp->val.dst_port) {
-        tcp_udp->mask.dst_port = FS_MASK_ON_16;
-    }
-}
-
-static inline void ibv_flow_spec_flow_tag_set(xlio_ibv_flow_spec_action_tag *flow_tag,
-                                              uint32_t tag_id)
-{
-    NOT_IN_USE(tag_id);
-    if (flow_tag == NULL) {
-        return;
-    }
-#ifdef DEFINED_IBV_FLOW_TAG
-    flow_tag->type = XLIO_IBV_FLOW_SPEC_ACTION_TAG;
-    flow_tag->size = sizeof(xlio_ibv_flow_spec_action_tag);
-    flow_tag->tag_id = tag_id;
-#endif // DEFINED_IBV_FLOW_TAG
-}
-
 #endif
