@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -38,23 +40,17 @@
 #include "core/event/event_handler_manager.h"
 #include "mem_buf_desc.h"
 
+#undef MODULE_NAME
+#define MODULE_NAME "ip_frag"
+
 //#define IP_FRAG_DEBUG 1
 
 #ifdef IP_FRAG_DEBUG
-#define frag_dbg(fmt, args...)                                                                     \
-    vlog_printf(VLOG_WARNING, "%s:%d : " fmt "\n", __FUNCTION__, __LINE__, ##args)
+#define frag_dbg __log_info_dbg
 #else
 #define frag_dbg(fmt, args...)
 #endif
-
-#define frag_err(fmt, args...)                                                                     \
-    vlog_printf(VLOG_ERROR, "%s:%d : " fmt "\n", __FUNCTION__, __LINE__, ##args)
-
-#define frag_panic(fmt, args...)                                                                   \
-    do {                                                                                           \
-        vlog_printf(VLOG_PANIC, "%s:%d : " fmt "\n", __FUNCTION__, __LINE__, ##args);              \
-        throw;                                                                                     \
-    } while (0)
+#define frag_panic __log_info_panic
 
 #ifdef IP_FRAG_DEBUG
 static int debug_drop_every_n_pkt = 0; // 0 - Disabled, 1/N is the number of packet dropped
@@ -64,9 +60,9 @@ static int g_ip_frag_count_check = 0;
 #define MEMBUF_DEBUG_REF_INC(__p_desc__)                                                           \
     {                                                                                              \
         g_ip_frag_count_check++;                                                                   \
-        if (__p_desc__->n_ref_count != 0)                                                          \
+        if (__p_desc__->inc_ref_count() != 0) {                                                    \
             frag_panic("REF_INC: p=%p\n", __p_desc__);                                             \
-        __p_desc__->n_ref_count++;                                                                 \
+        }                                                                                          \
     }
 #define MEMBUF_DEBUG_REF_DEC(__p_desc__)                                                           \
     {                                                                                              \
@@ -79,9 +75,9 @@ static int g_ip_frag_count_check = 0;
 #define MEMBUF_DEBUG_REF_DEC_1(__p_desc__)                                                         \
     {                                                                                              \
         g_ip_frag_count_check--;                                                                   \
-        __p_desc__->n_ref_count--;                                                                 \
-        if (__p_desc__->n_ref_count != 0)                                                          \
+        if (__p_desc__->dec_ref_count() != 1) {                                                    \
             frag_panic("REF_DEC: p=%p\n", __p_desc__);                                             \
+        }                                                                                          \
     }
 #define PRINT_STATISTICS()                                                                         \
     {                                                                                              \
@@ -93,14 +89,14 @@ static int g_ip_frag_count_check = 0;
 #define PRINT_STATISTICS()
 #endif
 
-ip_frag_manager *g_p_ip_frag_manager = NULL;
+ip_frag_manager *g_p_ip_frag_manager = nullptr;
 
-ip_frag_hole_desc *hole_base = NULL;
-ip_frag_hole_desc *hole_free_list_head = NULL;
+ip_frag_hole_desc *hole_base = nullptr;
+ip_frag_hole_desc *hole_free_list_head = nullptr;
 int hole_free_list_count = 0;
 
-ip_frag_desc *desc_base = NULL;
-ip_frag_desc *desc_free_list_head = NULL;
+ip_frag_desc *desc_base = nullptr;
+ip_frag_desc *desc_free_list_head = nullptr;
 int desc_free_list_count = 0;
 
 ip_frag_manager::ip_frag_manager()
@@ -162,7 +158,7 @@ void ip_frag_manager::free_frag_resources(void)
 
     unlock();
 
-    // Must call cq_mgr outside the lock to avoid ABBA deadlock
+    // Must call cq_mgr_rx outside the lock to avoid ABBA deadlock
     return_buffers_to_owners(temp_buff_map);
 
     delete[] desc_base;
@@ -215,7 +211,7 @@ ip_frag_hole_desc *ip_frag_manager::alloc_hole_desc()
     struct ip_frag_hole_desc *ret;
     ret = hole_free_list_head;
     if (!ret) {
-        return NULL;
+        return nullptr;
     }
 
     // unlink from hole's free list
@@ -223,9 +219,9 @@ ip_frag_hole_desc *ip_frag_manager::alloc_hole_desc()
     hole_free_list_count--;
 
     // clear hole struct
-    ret->data_first = 0;
-    ret->data_last = 0;
-    ret->next = 0;
+    ret->data_first = nullptr;
+    ret->data_last = nullptr;
+    ret->next = nullptr;
     return ret;
 }
 
@@ -242,14 +238,14 @@ ip_frag_desc_t *ip_frag_manager::alloc_frag_desc()
     ip_frag_desc_t *ret;
     ret = desc_free_list_head;
     if (!ret) {
-        return NULL;
+        return nullptr;
     }
 
     // unlink from hole's free list
     desc_free_list_head = ret->next;
     --desc_free_list_count;
 
-    ret->next = 0;
+    ret->next = nullptr;
     return ret;
 }
 
@@ -282,13 +278,13 @@ void ip_frag_manager::destroy_frag_desc(ip_frag_desc_t *desc)
  */
 ip_frag_desc_t *ip_frag_manager::new_frag_desc(ip_frag_key_t &key)
 {
-    ip_frag_desc_t *desc = NULL;
-    struct ip_frag_hole_desc *hole = NULL;
+    ip_frag_desc_t *desc = nullptr;
+    struct ip_frag_hole_desc *hole = nullptr;
 
     hole = alloc_hole_desc();
     if (!hole) {
         frag_dbg("NULL hole");
-        return NULL;
+        return nullptr;
     }
     hole->first = IP_FRAG_NINF;
     hole->last = IP_FRAG_INF;
@@ -297,10 +293,10 @@ ip_frag_desc_t *ip_frag_manager::new_frag_desc(ip_frag_key_t &key)
     if (!desc) {
         frag_dbg("NULL desc");
         free_hole_desc(hole);
-        return NULL;
+        return nullptr;
     }
     desc->ttl = IP_FRAG_TTL;
-    desc->frag_list = 0;
+    desc->frag_list = nullptr;
     desc->hole_list = hole;
     desc->frag_counter = m_frag_counter;
 
@@ -388,7 +384,7 @@ int ip_frag_manager::add_frag(iphdr *hdr, mem_buf_desc_t *frag, mem_buf_desc_t *
             frag_dbg("> old fragmented packet");
         }
     }
-    if (desc == NULL) {
+    if (!desc) {
         MEMBUF_DEBUG_REF_DEC(frag);
         PRINT_STATISTICS();
         unlock();
@@ -399,7 +395,7 @@ int ip_frag_manager::add_frag(iphdr *hdr, mem_buf_desc_t *frag, mem_buf_desc_t *
 
     /* 8 step reassembly algorithm as described in RFC 815 */
     // step 1
-    phole_prev = 0;
+    phole_prev = nullptr;
     phole = desc->hole_list;
     while (phole) {
         // step 2 and step 3
@@ -507,7 +503,7 @@ int ip_frag_manager::add_frag(iphdr *hdr, mem_buf_desc_t *frag, mem_buf_desc_t *
     }
     frag_dbg("> need more packets");
 
-    *ret = NULL;
+    *ret = nullptr;
     PRINT_STATISTICS();
     unlock();
     return 0;
@@ -565,7 +561,7 @@ void ip_frag_manager::handle_timer_expired(void *user_data)
     PRINT_STATISTICS();
     unlock();
 
-    // Must call cq_mgr outside the lock to avoid ABBA deadlock
+    // Must call cq_mgr_rx outside the lock to avoid ABBA deadlock
     return_buffers_to_owners(temp_buff_map);
 }
 
