@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -41,11 +43,11 @@
 #include "event/event_handler_manager.h"
 #include "event/timer_handler.h"
 #include "sock/cleanable_obj.h"
-#include "sock/socket_fd_api.h"
+#include "sock/sockinfo.h"
 #include "iomux/epfd_info.h"
 #include "utils/lock_wrapper.h"
 
-typedef xlio_list_t<socket_fd_api, socket_fd_api::pendig_to_remove_node_offset> sock_fd_api_list_t;
+typedef xlio_list_t<sockinfo, sockinfo::pending_to_remove_node_offset> sock_fd_api_list_t;
 typedef xlio_list_t<epfd_info, epfd_info::epfd_info_node_offset> epfd_info_list_t;
 
 typedef std::unordered_map<pthread_t, int> offload_thread_rule_t;
@@ -67,7 +69,7 @@ class cq_channel_info : public cleanable_obj {
 public:
     cq_channel_info(ring *p_ring)
         : m_p_ring(p_ring) {};
-    ~cq_channel_info() {};
+    ~cq_channel_info() override = default;
     ring *get_ring() const noexcept { return m_p_ring; };
 
 protected:
@@ -77,7 +79,7 @@ protected:
 class fd_collection : private lock_mutex_recursive {
 public:
     fd_collection();
-    ~fd_collection();
+    ~fd_collection() override;
 
     /**
      * Create and add a sockinfo. Use get_sock() to get it.
@@ -87,14 +89,6 @@ public:
      */
     int addsocket(int fd, int domain, int type, bool check_offload = false);
 
-    /**
-     * Create pipeinfo. Use get_sock() to get it.
-     * @param fdrd Read fd.
-     * @param fdwr Write fd.
-     * @return 0 on success, -1 on failure.
-     */
-    int addpipe(int fdrd, int fdwr);
-
     /**
      * Create epfd_info. Use get_epfd() to get it.
      * @param epfd epoll fd.
@@ -120,9 +114,9 @@ public:
     int addtapfd(int tapfd, ring_tap *p_ring);
 
     /**
-     * Remove pipeinfo/sockinfo.
+     * Remove sockinfo.
      */
-    int del_sockfd(int fd, bool b_cleanup = false, bool is_for_udp_pool = false);
+    int del_sockfd(int fd, bool is_for_udp_pool = false);
 
     /**
      * Remove epfd_info.
@@ -140,17 +134,20 @@ public:
      */
     void del_tapfd(int fd);
 
+    void set_socket(int fd, sockinfo *si) { m_p_sockfd_map[fd] = si; }
+    void clear_socket(int fd) { m_p_sockfd_map[fd] = nullptr; }
+
     /**
      * Call set_immediate_os_sample of the input fd.
      */
     inline bool set_immediate_os_sample(int fd);
 
-    inline void reuse_sockfd(int fd, socket_fd_api *p_sfd_api_obj);
-    inline void destroy_sockfd(socket_fd_api *p_sfd_api_obj);
+    inline void reuse_sockfd(int fd, sockinfo *p_sfd_api_obj);
+    inline void destroy_sockfd(sockinfo *p_sfd_api_obj);
     /**
-     * Get sock_fd_api (sockinfo or pipeinfo) by fd.
+     * Get sock_fd_api (sockinfo) by fd.
      */
-    inline socket_fd_api *get_sockfd(int fd);
+    inline sockinfo *get_sockfd(int fd);
 
     /**
      * Get epfd_info by fd.
@@ -192,15 +189,29 @@ public:
 
 #if defined(DEFINED_NGINX)
     bool pop_socket_pool(int &fd, bool &add_to_udp_pool, int type);
-    void push_socket_pool(socket_fd_api *sockfd);
+    void push_socket_pool(sockinfo *sockfd);
     void handle_socket_pool(int fd);
 #endif
 private:
     template <typename cls> int del(int fd, bool b_cleanup, cls **map_type);
     template <typename cls> inline cls *get(int fd, cls **map_type);
+    int del_socket(int fd, sockinfo **map_type);
+    inline bool is_valid_fd(int fd);
 
+    inline bool create_offloaded_sockets();
+
+    // Fd collection timer implementation
+    // This gives context to handle pending to remove fds.
+    // In case of TCP we recheck if TCP socket is closable and delete
+    // it if it does otherwise we run handle_timer of the socket to
+    // progress the TCP connection.
+    void handle_timer_expired(void *user_data);
+
+    void statistics_print_helper(int fd, vlog_levels_t log_level);
+
+private:
     int m_n_fd_map_size;
-    socket_fd_api **m_p_sockfd_map;
+    sockinfo **m_p_sockfd_map;
     epfd_info **m_p_epfd_map;
     cq_channel_info **m_p_cq_channel_map;
     ring_tap **m_p_tap_map;
@@ -215,22 +226,9 @@ private:
     // else contain all threads that need to be offloaded.
     offload_thread_rule_t m_offload_thread_rule;
 
-    inline bool is_valid_fd(int fd);
-
-    inline bool create_offloaded_sockets();
-
-    // Fd collection timer implementation
-    // This gives context to handle pending to remove fds.
-    // In case of TCP we recheck if TCP socket is closable and delete
-    // it if it does otherwise we run handle_timer of the socket to
-    // progress the TCP connection.
-    void handle_timer_expired(void *user_data);
-
-    void statistics_print_helper(int fd, vlog_levels_t log_level);
-
 #if defined(DEFINED_NGINX)
     bool m_use_socket_pool;
-    std::stack<socket_fd_api *> m_socket_pool;
+    std::stack<sockinfo *> m_socket_pool;
     int m_socket_pool_size;
     int m_socket_pool_counter;
 #endif
@@ -256,7 +254,6 @@ template <typename cls> inline cls *fd_collection::get(int fd, cls **map_type)
 
 inline bool fd_collection::set_immediate_os_sample(int fd)
 {
-    epfd_info *epfd_fd;
     ring_tap *p_ring;
 
     lock();
@@ -267,17 +264,11 @@ inline bool fd_collection::set_immediate_os_sample(int fd)
         return true;
     }
 
-    if ((epfd_fd = get_epfd(fd))) {
-        epfd_fd->set_os_data_available();
-        unlock();
-        return true;
-    }
-
     unlock();
     return false;
 }
 
-inline void fd_collection::reuse_sockfd(int fd, socket_fd_api *p_sfd_api_obj)
+inline void fd_collection::reuse_sockfd(int fd, sockinfo *p_sfd_api_obj)
 {
     lock();
     m_pending_to_remove_lst.erase(p_sfd_api_obj);
@@ -286,16 +277,16 @@ inline void fd_collection::reuse_sockfd(int fd, socket_fd_api *p_sfd_api_obj)
     unlock();
 }
 
-inline void fd_collection::destroy_sockfd(socket_fd_api *p_sfd_api_obj)
+inline void fd_collection::destroy_sockfd(sockinfo *p_sfd_api_obj)
 {
     lock();
     --g_global_stat_static.n_pending_sockets;
     m_pending_to_remove_lst.erase(p_sfd_api_obj);
-    p_sfd_api_obj->clean_obj();
+    p_sfd_api_obj->clean_socket_obj();
     unlock();
 }
 
-inline socket_fd_api *fd_collection::get_sockfd(int fd)
+inline sockinfo *fd_collection::get_sockfd(int fd)
 {
     return get(fd, m_p_sockfd_map);
 }
@@ -322,12 +313,12 @@ inline int fd_collection::get_fd_map_size()
 
 extern fd_collection *g_p_fd_collection;
 
-inline socket_fd_api *fd_collection_get_sockfd(int fd)
+inline sockinfo *fd_collection_get_sockfd(int fd)
 {
     if (g_p_fd_collection) {
         return g_p_fd_collection->get_sockfd(fd);
     }
-    return NULL;
+    return nullptr;
 }
 
 inline epfd_info *fd_collection_get_epfd(int fd)
@@ -335,7 +326,7 @@ inline epfd_info *fd_collection_get_epfd(int fd)
     if (g_p_fd_collection) {
         return g_p_fd_collection->get_epfd(fd);
     }
-    return NULL;
+    return nullptr;
 }
 
 #endif
