@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -133,7 +135,7 @@ int get_base_interface_name(const char *if_name, char *base_ifname, size_t sz_ba
         }
         BULLSEYE_EXCLUDE_BLOCK_END
 
-        for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
+        for (ifa = ifaddr; ifa; ifa = ifa->ifa_next) {
             if (!strcmp(ifa->ifa_name, if_name)) {
                 continue;
             }
@@ -205,6 +207,20 @@ void print_roce_lag_warnings(const char *interface, char *disable_path /* = NULL
                 "********************\n");
 }
 
+void print_warning_rlimit_memlock(size_t length, int error)
+{
+    vlog_printf(VLOG_ERROR,
+                "**********************************************************************************"
+                "********************\n");
+    vlog_printf(VLOG_ERROR, "* Failed registering a memory region of size %zu bytes\n", length);
+    vlog_printf(VLOG_ERROR, "* (errno=%d %m)\n", error);
+    vlog_printf(VLOG_ERROR, "* Could be due to lack of locked memory in kernel.\n");
+    vlog_printf(VLOG_ERROR, "* Please check max allowed locked memory (ulimit -l)\n");
+    vlog_printf(VLOG_ERROR,
+                "**********************************************************************************"
+                "********************\n");
+}
+
 void compute_tx_checksum(mem_buf_desc_t *p_mem_buf_desc, bool l3_csum, bool l4_csum)
 {
     unsigned short l3_checksum = -1, l4_checksum = -1;
@@ -351,7 +367,9 @@ unsigned short compute_udp_checksum_tx(const ip6_hdr *ipv6, const uint16_t *payl
 {
     uint16_t ipLen = ntohs(ipv6->ip6_plen);
     uint32_t sum = compute_pseudo_header(ipv6, IPPROTO_UDP, ntohs(udp->len));
-    return compute_payload_checksum(payload, ipLen, sum);
+
+    // For UDP, checksum zero means no checksum. Zero must be replaced with 0xffff.
+    return compute_payload_checksum(payload, ipLen, sum) ?: 0xffff;
 }
 
 /*
@@ -370,7 +388,11 @@ unsigned short compute_ipv6_udp_frag_checksum(const ip6_hdr *ipv6, udphdr *udp)
         sum = (sum & 0xffff) + (sum >> 16);
     }
 
-    sum = ~sum;
+    // For UDP, checksum zero means no checksum. Zero must be replaced with 0xffff.
+    if (sum != 0xffff) {
+        sum = ~sum;
+    }
+
     return static_cast<unsigned short>(sum);
 }
 
@@ -475,7 +497,7 @@ void set_fd_block_mode(int fd, bool b_block)
 {
     __log_dbg("fd[%d]: setting to %sblocking mode (%d)", fd, b_block ? "" : "non-", b_block);
 
-    int flags = orig_os_api.fcntl(fd, F_GETFL);
+    int flags = SYSCALL(fcntl, fd, F_GETFL);
     BULLSEYE_EXCLUDE_BLOCK_START
     if (flags < 0) {
         __log_err("failed reading fd[%d] flag (rc=%d errno=%d %m)", fd, flags, errno);
@@ -489,7 +511,7 @@ void set_fd_block_mode(int fd, bool b_block)
         flags |= O_NONBLOCK;
     }
 
-    int ret = orig_os_api.fcntl(fd, F_SETFL, flags);
+    int ret = SYSCALL(fcntl, fd, F_SETFL, flags);
     BULLSEYE_EXCLUDE_BLOCK_START
     if (ret < 0) {
         __log_err("failed changing fd[%d] to %sblocking mode (rc=%d errno=%d %s)", fd,
@@ -524,32 +546,34 @@ int priv_read_file(const char *path, char *buf, size_t size,
                    vlog_levels_t log_level /*= VLOG_ERROR*/)
 {
     int len = -1;
-    int fd = open(path, O_RDONLY);
+    int fd = SYSCALL(open, path, O_RDONLY);
     BULLSEYE_EXCLUDE_BLOCK_START
     if (fd < 0) {
-        VLOG_PRINTF(log_level, "ERROR while opening file %s (errno %d %m)", path, errno);
+        VLOG_PRINTF(log_level, "Couldn't open file %s (errno %d %m)", path, errno);
         return -1;
     }
     BULLSEYE_EXCLUDE_BLOCK_END
-    len = read(fd, buf, size);
+    len = SYSCALL(read, fd, buf, size);
     BULLSEYE_EXCLUDE_BLOCK_START
     if (len < 0) {
-        VLOG_PRINTF(log_level, "ERROR while reading from file %s (errno %d %m)", path, errno);
+        VLOG_PRINTF(log_level, "Couldn't read file %s (errno %d %m)", path, errno);
     }
     BULLSEYE_EXCLUDE_BLOCK_END
-    close(fd);
+    SYSCALL(close, fd);
     return len;
 }
 
-int read_file_to_int(const char *path, int default_value, vlog_levels_t log_level)
+int read_file_to_int(const char *path, int default_value, vlog_levels_t log_level /*VLOG_WARNING*/)
 {
-    char buf[25];
-    int rc = priv_safe_read_file(path, buf, sizeof buf, log_level);
-    if (rc < 0) {
-        VLOG_PRINTF(log_level, "ERROR while getting int from from file %s, we'll use default %d",
-                    path, default_value);
+    int value = -1;
+    std::ifstream file_stream(path);
+    if (!file_stream || !(file_stream >> value)) {
+        VLOG_PRINTF(log_level,
+                    "Couldn't read an integer from file %s (errno %d %m), we'll use default %d",
+                    path, errno, default_value);
+        return default_value;
     }
-    return (rc < 0) ? default_value : atoi(buf);
+    return value;
 }
 
 int get_port_from_ifname(const char *ifname)
@@ -562,13 +586,13 @@ int get_port_from_ifname(const char *ifname)
     snprintf(dev_path, sizeof(dev_path), VERBS_DEVICE_PORT_PARAM_FILE, ifname);
     if (priv_safe_try_read_file(dev_path, num_buf, sizeof(num_buf)) > 0) {
         dev_port =
-            strtol(num_buf, NULL, 0); // base=0 means strtol() can parse hexadecimal and decimal
+            strtol(num_buf, nullptr, 0); // base=0 means strtol() can parse hexadecimal and decimal
         __log_dbg("dev_port file=%s dev_port str=%s dev_port val=%d", dev_path, num_buf, dev_port);
     }
     snprintf(dev_path, sizeof(dev_path), VERBS_DEVICE_ID_PARAM_FILE, ifname);
     if (priv_safe_try_read_file(dev_path, num_buf, sizeof(num_buf)) > 0) {
         dev_id =
-            strtol(num_buf, NULL, 0); // base=0 means strtol() can parse hexadecimal and decimal
+            strtol(num_buf, nullptr, 0); // base=0 means strtol() can parse hexadecimal and decimal
         __log_dbg("dev_id file= %s dev_id str=%s dev_id val=%d", dev_path, num_buf, dev_id);
     }
 
@@ -584,7 +608,7 @@ int get_iftype_from_ifname(const char *ifname)
     char iftype_filename[100];
     char iftype_value_str[32];
     char base_ifname[32];
-    char iftype_value = -1;
+    int iftype_value = -1;
 
     get_base_interface_name(ifname, base_ifname, sizeof(base_ifname));
     sprintf(iftype_filename, IFTYPE_PARAM_FILE, base_ifname);
@@ -653,20 +677,20 @@ public:
             .tv_usec = 10,
         };
 
-        m_fd = orig_os_api.socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
+        m_fd = SYSCALL(socket, AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
         if (m_fd < 0) {
             throw std::runtime_error("Open netlink socket failed");
         }
 
-        if (orig_os_api.setsockopt(m_fd, SOL_SOCKET, SO_RCVTIMEO, (const char *)&tv, sizeof tv)) {
-            close(m_fd);
+        if (SYSCALL(setsockopt, m_fd, SOL_SOCKET, SO_RCVTIMEO, (const char *)&tv, sizeof tv)) {
+            SYSCALL(close, m_fd);
             throw std::runtime_error("Setsockopt non-blocking failed");
         }
     }
 
     socket_context_manager(int fd) noexcept
         : m_fd(fd) {};
-    ~socket_context_manager() { close(m_fd); };
+    ~socket_context_manager() { SYSCALL(close, m_fd); };
 
     void send_getaddr_request(uint8_t family)
     {
@@ -688,7 +712,7 @@ public:
         iovec iov = {&msg_buf, msg_buf.nl.nlmsg_len};
         msghdr msg = {&sa, sizeof(sa), &iov, 1, nullptr, 0, 0};
 
-        if (orig_os_api.sendmsg(m_fd, &msg, 0) < 0) {
+        if (SYSCALL(sendmsg, m_fd, &msg, 0) < 0) {
             throw std::runtime_error("Send RTM_GETADDR request failed");
         }
     }
@@ -699,7 +723,7 @@ public:
         iovec iov = {&m_buf, m_buf.size()};
         msghdr msg = {&sa, sizeof(sa), &iov, 1, nullptr, 0, 0};
 
-        return orig_os_api.recvmsg(m_fd, &msg, 0);
+        return SYSCALL(recvmsg, m_fd, &msg, 0);
     }
 
     nlmsghdr *get_nlmsghdr() { return reinterpret_cast<nlmsghdr *>(&m_buf); }
@@ -816,7 +840,7 @@ uint16_t get_vlan_id_from_ifname(const char *ifname)
 {
     // find vlan id from interface name
     struct vlan_ioctl_args ifr;
-    int fd = orig_os_api.socket(AF_INET, SOCK_DGRAM, 0);
+    int fd = SYSCALL(socket, AF_INET, SOCK_DGRAM, 0);
 
     if (fd < 0) {
         __log_err("ERROR from socket() (errno=%d %m)", errno);
@@ -826,15 +850,15 @@ uint16_t get_vlan_id_from_ifname(const char *ifname)
     ifr.cmd = GET_VLAN_VID_CMD;
     strncpy(ifr.device1, ifname, sizeof(ifr.device1) - 1);
 
-    if (orig_os_api.ioctl(fd, SIOCGIFVLAN, &ifr) < 0) {
+    if (SYSCALL(ioctl, fd, SIOCGIFVLAN, &ifr) < 0) {
         __log_dbg(
             "Failure in ioctl(SIOCGIFVLAN, cmd=GET_VLAN_VID_CMD) for interface '%s' (errno=%d %m)",
             ifname, errno);
-        orig_os_api.close(fd);
+        SYSCALL(close, fd);
         return 0;
     }
 
-    orig_os_api.close(fd);
+    SYSCALL(close, fd);
 
     __log_dbg("found vlan id '%d' for interface '%s'", ifr.u.VID, ifname);
 
@@ -845,7 +869,7 @@ size_t get_vlan_base_name_from_ifname(const char *ifname, char *base_ifname, siz
 {
     // find vlan base name from interface name
     struct vlan_ioctl_args ifr;
-    int fd = orig_os_api.socket(AF_INET, SOCK_DGRAM, 0);
+    int fd = SYSCALL(socket, AF_INET, SOCK_DGRAM, 0);
     if (fd < 0) {
         __log_err("ERROR from socket() (errno=%d %m)", errno);
         return -1;
@@ -854,15 +878,15 @@ size_t get_vlan_base_name_from_ifname(const char *ifname, char *base_ifname, siz
     ifr.cmd = GET_VLAN_REALDEV_NAME_CMD;
     strncpy(ifr.device1, ifname, sizeof(ifr.device1) - 1);
 
-    if (orig_os_api.ioctl(fd, SIOCGIFVLAN, &ifr) < 0) {
+    if (SYSCALL(ioctl, fd, SIOCGIFVLAN, &ifr) < 0) {
         __log_dbg("Failure in ioctl(SIOCGIFVLAN, cmd=GET_VLAN_REALDEV_NAME_CMD) for interface '%s' "
                   "(errno=%d %m)",
                   ifname, errno);
-        orig_os_api.close(fd);
+        SYSCALL(close, fd);
         return 0;
     }
 
-    orig_os_api.close(fd);
+    SYSCALL(close, fd);
 
     size_t name_len = strlen(ifr.u.device2);
     if (base_ifname && name_len > 0) {
@@ -903,7 +927,7 @@ int run_and_retreive_system_command(const char *cmd_line, char *return_str, int
     if (file) {
         int fd = fileno(file);
         if (fd > 0) {
-            int actual_len = read(fd, return_str, return_str_len - 1);
+            int actual_len = SYSCALL(read, fd, return_str, return_str_len - 1);
             if (actual_len > 0) {
                 return_str[actual_len] = '\0';
             } else {
@@ -970,9 +994,9 @@ size_t get_local_ll_addr(IN const char *ifname, OUT unsigned char *addr, IN int
 bool check_bond_device_exist(const char *ifname)
 {
     int ret = 0;
-    struct nl_cache *cache = NULL;
-    struct rtnl_link *link = NULL;
-    char *link_type = NULL;
+    struct nl_cache *cache = nullptr;
+    struct rtnl_link *link = nullptr;
+    char *link_type = nullptr;
 
     struct nl_sock *nl_socket = nl_socket_alloc();
     if (!nl_socket) {
@@ -993,7 +1017,7 @@ bool check_bond_device_exist(const char *ifname)
     }
     link_type = rtnl_link_get_type(link);
     if (link_type && (strcmp(link_type, "bond") != 0)) {
-        link_type = NULL;
+        link_type = nullptr;
     }
 out:
     if (link) {
@@ -1022,12 +1046,12 @@ bool get_bond_name(IN const char *ifname, OUT char *bond_name, IN int sz)
         return ret;
     }
 
-    for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
+    for (ifa = ifaddr; ifa; ifa = ifa->ifa_next) {
         snprintf(upper_path, sizeof(upper_path), NETVSC_DEVICE_UPPER_FILE, base_ifname,
                  ifa->ifa_name);
-        int fd = open(upper_path, O_RDONLY);
+        int fd = SYSCALL(open, upper_path, O_RDONLY);
         if (fd >= 0) {
-            close(fd);
+            SYSCALL(close, fd);
             if (IFNAMSIZ <= sz) {
                 memcpy(bond_name, ifa->ifa_name, IFNAMSIZ);
             }
@@ -1085,12 +1109,12 @@ bool get_netvsc_slave(IN const char *ifname, OUT char *slave_name, OUT unsigned
         return ret;
     }
 
-    for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
+    for (ifa = ifaddr; ifa; ifa = ifa->ifa_next) {
         snprintf(netvsc_path, sizeof(netvsc_path), NETVSC_DEVICE_LOWER_FILE, base_ifname,
                  ifa->ifa_name);
-        int fd = open(netvsc_path, O_RDONLY);
+        int fd = SYSCALL(open, netvsc_path, O_RDONLY);
         if (fd >= 0) {
-            close(fd);
+            SYSCALL(close, fd);
             memcpy(slave_name, ifa->ifa_name, IFNAMSIZ);
             slave_flags = ifa->ifa_flags;
             __log_dbg("Found slave_name = %s, slave_flags = %u", slave_name, slave_flags);
@@ -1165,9 +1189,9 @@ bool check_device_exist(const char *ifname, const char *path)
 
     n = snprintf(device_path, sizeof(device_path), path, ifname);
     if (likely((0 < n) && (n < (int)sizeof(device_path)))) {
-        fd = orig_os_api.open(device_path, O_RDONLY);
+        fd = SYSCALL(open, device_path, O_RDONLY);
         if (fd >= 0) {
-            orig_os_api.close(fd);
+            SYSCALL(close, fd);
         }
         if (fd < 0 && errno == EMFILE) {
             __log_warn("There are no free fds in the system. This may cause unexpected behavior");
@@ -1191,9 +1215,9 @@ bool check_device_name_ib_name(const char *ifname, const char *ibname)
     n = snprintf(ib_path, sizeof(ib_path), "/sys/class/infiniband/%s/device/net/%s/ifindex", ibname,
                  str_ifname);
     if (likely((0 < n) && (n < (int)sizeof(ib_path)))) {
-        fd = open(ib_path, O_RDONLY);
+        fd = SYSCALL(open, ib_path, O_RDONLY);
         if (fd >= 0) {
-            close(fd);
+            SYSCALL(close, fd);
             return true;
         }
     }
@@ -1274,34 +1298,6 @@ bool validate_user_has_cap_net_raw_privliges()
 #endif
 }
 
-size_t default_huge_page_size(void)
-{
-    static size_t hugepage_sz = 0;
-#ifdef MAP_HUGETLB
-
-    if (!hugepage_sz) {
-        char str[1024];
-        unsigned long sz;
-        FILE *file;
-
-        file = fopen("/proc/meminfo", "rt");
-        if (file) {
-            while (fgets(str, sizeof(str), file) != NULL) {
-                if (sscanf(str, "Hugepagesize:   %8lu kB", &sz) == 1) {
-                    hugepage_sz = sz * 1024;
-                    break;
-                }
-            }
-            fclose(file);
-        }
-    }
-#endif
-
-    __log_dbg("Detect default Hugepage size: %zd", hugepage_sz);
-
-    return hugepage_sz;
-}
-
 int validate_tso(int if_index)
 {
 #ifdef HAVE_LINUX_ETHTOOL_H
@@ -1310,7 +1306,7 @@ int validate_tso(int if_index)
     struct ifreq req;
     struct ethtool_value eval;
 
-    fd = orig_os_api.socket(AF_INET, SOCK_DGRAM, 0);
+    fd = SYSCALL(socket, AF_INET, SOCK_DGRAM, 0);
     if (fd < 0) {
         __log_err("ERROR from socket() (errno=%d %m)", errno);
         return -1;
@@ -1320,13 +1316,13 @@ int validate_tso(int if_index)
     req.ifr_ifindex = if_index;
     if_indextoname(if_index, req.ifr_name);
     req.ifr_data = (char *)&eval;
-    ret = orig_os_api.ioctl(fd, SIOCETHTOOL, &req);
+    ret = SYSCALL(ioctl, fd, SIOCETHTOOL, &req);
     if (ret < 0) {
         __log_dbg("ioctl(SIOCETHTOOL) cmd=ETHTOOL_GTSO (errno=%d %m)", errno);
     } else {
         ret = eval.data;
     }
-    orig_os_api.close(fd);
+    SYSCALL(close, fd);
     return ret;
 #else
     NOT_IN_USE(if_index);
@@ -1342,7 +1338,7 @@ int validate_lro(int if_index)
     struct ifreq req;
     struct ethtool_value eval;
 
-    fd = orig_os_api.socket(AF_INET, SOCK_DGRAM, 0);
+    fd = SYSCALL(socket, AF_INET, SOCK_DGRAM, 0);
     if (fd < 0) {
         __log_err("ERROR from socket() (errno=%d %m)", errno);
         return -1;
@@ -1352,13 +1348,13 @@ int validate_lro(int if_index)
     req.ifr_ifindex = if_index;
     if_indextoname(if_index, req.ifr_name);
     req.ifr_data = (char *)&eval;
-    ret = orig_os_api.ioctl(fd, SIOCETHTOOL, &req);
+    ret = SYSCALL(ioctl, fd, SIOCETHTOOL, &req);
     if (ret < 0) {
         __log_dbg("ioctl(SIOCETHTOOL) cmd=ETHTOOL_GFLAGS (errno=%d %m)", errno);
     } else {
         ret = (eval.data & ETH_FLAG_LRO ? 1 : 0);
     }
-    orig_os_api.close(fd);
+    SYSCALL(close, fd);
     return ret;
 #else
     NOT_IN_USE(if_index);
@@ -1418,8 +1414,9 @@ xlio_error::xlio_error(const char *_message, const char *_function, const char *
     , lineno(_lineno)
     , errnum(_errnum)
 {
-    snprintf(formatted_message, sizeof(formatted_message), "xlio_error <%s> (errno=%d %s) in %s:%d",
-             message, errnum, strerror(errnum), filename, lineno);
+    snprintf(formatted_message, sizeof(formatted_message),
+             "xlio_error <%s> (errno=%d %s) in %s:%d\n", message, errnum, strerror(errnum),
+             filename, lineno);
     formatted_message[sizeof(formatted_message) - 1] = '\0';
 }
 
