@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -48,15 +50,11 @@ public:
     void going_to_sleep();
     void return_from_sleep() { --m_is_sleeping; };
     void wakeup_clear() { m_is_sleeping = 0; }
+    void wakeup_set_epoll_fd(int epfd);
 
 protected:
-    virtual void wakeup_set_epoll_fd(int epfd);
     int m_is_sleeping;
-
-    // lock_spin_recursive m_wakeup_lock; This lock is not needed for now. Maybe we will need it for
-    // epoll.
-
-    int m_epfd;
+    int m_wakeup_epfd;
     struct epoll_event m_ev;
 };
 
