@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -45,6 +47,7 @@
 #include <core/xlio_extra.h>
 #include <core/util/sock_addr.h>
 #include <assert.h>
+#include <atomic>
 
 #define NUM_OF_SUPPORTED_CQS         16
 #define NUM_OF_SUPPORTED_RINGS       16
@@ -104,6 +107,7 @@ struct user_params_t {
     int fd_dump;
     vlog_levels_t fd_dump_log_level;
     std::string xlio_stats_path;
+    std::ofstream csv_stream;
 };
 
 extern user_params_t user_params;
@@ -188,54 +192,82 @@ typedef struct {
     uint32_t n_strq_max_strides_per_packet;
 } socket_strq_counters_t;
 
-typedef struct {
+typedef struct socket_listen_counters {
     uint32_t n_rx_syn;
     uint32_t n_rx_syn_tw;
     uint32_t n_rx_fin;
     uint32_t n_conn_established;
     uint32_t n_conn_accepted;
     uint32_t n_conn_dropped;
-    uint32_t n_conn_backlog;
+    int n_conn_backlog;
+
+    socket_listen_counters() = default;
+    ~socket_listen_counters() = default;
+
+    socket_listen_counters &operator+=(const socket_listen_counters &rhs)
+    {
+        n_rx_syn += rhs.n_rx_syn;
+        n_rx_syn_tw += rhs.n_rx_syn_tw;
+        n_rx_fin += rhs.n_rx_fin;
+        n_conn_established += rhs.n_conn_established;
+        n_conn_accepted += rhs.n_conn_accepted;
+        n_conn_dropped += rhs.n_conn_dropped;
+        n_conn_backlog += rhs.n_conn_backlog;
+        return *this;
+    }
+    socket_listen_counters operator-(const socket_listen_counters &rhs)
+    {
+        return {n_rx_syn - rhs.n_rx_syn,
+                n_rx_syn_tw - rhs.n_rx_syn_tw,
+                n_rx_fin - rhs.n_rx_fin,
+                n_conn_established - rhs.n_conn_established,
+                n_conn_accepted - rhs.n_conn_accepted,
+                n_conn_dropped - rhs.n_conn_dropped,
+                n_conn_backlog - rhs.n_conn_backlog};
+    }
 } socket_listen_counters_t;
 
-typedef struct socket_stats_t {
-    int fd;
-    uint32_t inode;
-    uint32_t tcp_state; // enum tcp_state
-    uint8_t socket_type; // SOCK_STREAM, SOCK_DGRAM, ...
-    bool padding1;
-    sa_family_t sa_family;
-    bool b_is_offloaded;
-    bool b_blocking;
-    bool b_mc_loop;
-    bool padding2;
-    in_port_t bound_port;
-    in_port_t connected_port;
-    ip_address bound_if;
-    ip_address connected_ip;
-    ip_address mc_tx_if;
-    pid_t threadid_last_rx;
-    pid_t threadid_last_tx;
-    uint32_t n_rx_ready_pkt_count;
-    uint32_t n_rx_ready_byte_limit;
-    uint64_t n_rx_ready_byte_count;
+struct socket_stats_t {
+    // Data Path
     uint64_t n_tx_ready_byte_count;
-    uint32_t n_rx_zcopy_pkt_count;
+    uint64_t n_rx_ready_byte_count;
+    uint32_t n_rx_ready_pkt_count;
     socket_counters_t counters;
+    socket_strq_counters_t strq_counters;
 #ifdef DEFINED_UTLS
-    bool tls_tx_offload;
-    bool tls_rx_offload;
-    uint16_t tls_version;
-    uint16_t tls_cipher;
     socket_tls_counters_t tls_counters;
 #endif /* DEFINED_UTLS */
-    socket_strq_counters_t strq_counters;
     socket_listen_counters_t listen_counters;
+
+    // Control Path
     std::bitset<MC_TABLE_SIZE> mc_grp_map;
     ring_logic_t ring_alloc_logic_rx;
     ring_logic_t ring_alloc_logic_tx;
+    ip_address bound_if;
+    ip_address connected_ip;
+    ip_address mc_tx_if;
+    int fd;
+    uint32_t inode;
+    uint32_t tcp_state; // enum tcp_state
+    uint32_t n_rx_zcopy_pkt_count;
+    pid_t threadid_last_rx;
+    pid_t threadid_last_tx;
     uint64_t ring_user_id_rx;
     uint64_t ring_user_id_tx;
+    sa_family_t sa_family;
+    in_port_t bound_port;
+    in_port_t connected_port;
+    uint8_t socket_type; // SOCK_STREAM, SOCK_DGRAM, ...
+    bool b_is_offloaded;
+    bool b_blocking;
+    bool b_mc_loop;
+#ifdef DEFINED_UTLS
+    uint16_t tls_version;
+    uint16_t tls_cipher;
+    bool tls_tx_offload;
+    bool tls_rx_offload;
+#endif /* DEFINED_UTLS */
+    socket_stats_t *_next_stat;
 
     void reset()
     {
@@ -247,8 +279,8 @@ typedef struct socket_stats_t {
         bound_if = connected_ip = mc_tx_if = ip_address(in6addr_any);
         bound_port = connected_port = (in_port_t)0;
         threadid_last_rx = threadid_last_tx = pid_t(0);
-        n_rx_ready_pkt_count = n_rx_ready_byte_count = n_rx_ready_byte_limit =
-            n_rx_zcopy_pkt_count = n_tx_ready_byte_count = 0;
+        n_rx_ready_pkt_count = n_rx_ready_byte_count = n_rx_zcopy_pkt_count =
+            n_tx_ready_byte_count = 0;
         memset(&counters, 0, sizeof(counters));
 #ifdef DEFINED_UTLS
         tls_tx_offload = tls_rx_offload = false;
@@ -260,7 +292,6 @@ typedef struct socket_stats_t {
         mc_grp_map.reset();
         ring_user_id_rx = ring_user_id_tx = 0;
         ring_alloc_logic_rx = ring_alloc_logic_tx = RING_LOGIC_PER_INTERFACE;
-        padding1 = padding2 = 0;
     };
 
     void set_bound_if(sock_addr &sock)
@@ -285,10 +316,11 @@ typedef struct socket_stats_t {
         : bound_if(in6addr_any)
         , connected_ip(in6addr_any)
         , mc_tx_if(in6addr_any)
+        , _next_stat(nullptr)
     {
         reset();
     };
-} socket_stats_t;
+};
 
 typedef struct {
     bool b_enabled;
@@ -343,6 +375,8 @@ typedef struct {
     ring_type_t n_type;
     union {
         struct {
+            uint64_t n_tx_tso_pkt_count;
+            uint64_t n_tx_tso_byte_count;
             uint64_t n_rx_interrupt_requests;
             uint64_t n_rx_interrupt_received;
             uint32_t n_rx_cq_moderation_count;
@@ -352,6 +386,8 @@ typedef struct {
             uint64_t n_tx_dev_mem_byte_count;
             uint64_t n_tx_dev_mem_oob;
             uint32_t n_tx_dev_mem_allocated;
+            uint32_t n_tx_num_bufs;
+            uint32_t n_zc_num_bufs;
         } simple;
         struct {
             char s_tap_name[IFNAMSIZ];
@@ -374,6 +410,7 @@ typedef struct {
     uint32_t n_buffer_pool_size;
     uint32_t n_buffer_pool_no_bufs;
     uint32_t n_buffer_pool_expands;
+    uint32_t n_buffer_pool_created;
 } bpool_stats_t;
 
 typedef struct {
@@ -385,12 +422,27 @@ typedef struct {
 typedef struct {
     uint32_t n_tcp_seg_pool_size;
     uint32_t n_tcp_seg_pool_no_segs;
-    uint32_t n_pending_sockets;
+    int n_pending_sockets;
+    std::atomic<int> socket_tcp_destructor_counter;
+    std::atomic<int> socket_udp_destructor_counter;
+    void init()
+    {
+        n_tcp_seg_pool_size = 0;
+        n_tcp_seg_pool_no_segs = 0;
+        n_pending_sockets = 0;
+        socket_tcp_destructor_counter = 0;
+        socket_udp_destructor_counter = 0;
+    }
 } global_stats_t;
 
 typedef struct {
     bool b_enabled;
     global_stats_t global_stats;
+    void init()
+    {
+        b_enabled = false;
+        global_stats.init();
+    };
 } global_instance_block_t;
 
 // Version info
@@ -457,10 +509,12 @@ typedef struct sh_mem_t {
         memset(cq_inst_arr, 0, sizeof(cq_inst_arr));
         memset(ring_inst_arr, 0, sizeof(ring_inst_arr));
         memset(bpool_inst_arr, 0, sizeof(bpool_inst_arr));
-        memset(global_inst_arr, 0, sizeof(global_inst_arr));
+        global_inst_arr->init();
         mc_info.max_grp_num = 0;
         for (uint32_t i = 0; i < MC_TABLE_SIZE; i++) {
+            // coverity[missing_lock:FALSE] /*Turn off coverity missing_lock check*/
             mc_info.mc_grp_tbl[i].mc_grp = {ip_address::any_addr(), 0};
+            // coverity[missing_lock:FALSE]
             mc_info.mc_grp_tbl[i].sock_num = 0;
         }
         memset(&iomux, 0, sizeof(iomux));
