@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -44,6 +46,10 @@
 #include <errno.h>
 #include <dirent.h>
 #include <string.h>
+#include <ctime>
+#include <iomanip>
+#include <vector>
+#include <cinttypes>
 #include <sys/mman.h>
 #include <sys/resource.h>
 #include <sys/utsname.h>
@@ -59,6 +65,8 @@
 #include "core/util/utils.h"
 #include "core/util/xlio_stats.h"
 #include "core/util/sys_vars.h"
+#include "stats/stats_data_reader.h"
+#include <sstream>
 
 using namespace std;
 
@@ -99,6 +107,7 @@ typedef enum { e_K = 1024, e_M = 1048576 } units_t;
 #define CYCLES_SEPARATOR                                                                           \
     "-------------------------------------------------------------------------------\n"
 #define FORMAT_STATS_32bit     "%-20s %u\n"
+#define FORMAT_STATS_s_32bit   "%-20s %d\n"
 #define FORMAT_STATS_64bit     "%-20s %" PRIu64 " %-3s\n"
 #define FORMAT_STATS_double    "%-20s %.1f\n"
 #define FORMAT_RING_PACKETS    "%-20s %zu / %zu [kilobytes/packets] %-3s\n"
@@ -126,25 +135,6 @@ typedef enum { e_K = 1024, e_M = 1048576 } units_t;
 #define SEC_TO_MICRO(n)         ((n)*1000000)
 #define TIME_DIFF_in_MICRO(start, end)                                                             \
     (SEC_TO_MICRO((end).tv_sec - (start).tv_sec) + (NANO_TO_MICRO((end).tv_nsec - (start).tv_nsec)))
-// printf formating when IP is in network byte ordering (for LITTLE_ENDIAN)
-#define NETWORK_IP_PRINTQUAD_LITTLE_ENDIAN(ip)                                                     \
-    (uint8_t)((ip)&0xff), (uint8_t)(((ip) >> 8) & 0xff), (uint8_t)(((ip) >> 16) & 0xff),           \
-        (uint8_t)(((ip) >> 24) & 0xff)
-
-// printf formating when IP is in host byte ordering (for LITTLE_ENDIAN)
-#define HOST_IP_PRINTQUAD_LITTLE_ENDIAN(ip)                                                        \
-    (uint8_t)(((ip) >> 24) & 0xff), (uint8_t)(((ip) >> 16) & 0xff), (uint8_t)(((ip) >> 8) & 0xff), \
-        (uint8_t)((ip)&0xff)
-
-#if __BYTE_ORDER == __LITTLE_ENDIAN
-/* The host byte order is the same as network byte order, so these functions are all just identity.
- */
-#define NIPQUAD(ip) NETWORK_IP_PRINTQUAD_LITTLE_ENDIAN(ip)
-#else
-#if __BYTE_ORDER == __BIG_ENDIAN
-#define NIPQUAD(ip) HOST_IP_PRINTQUAD_LITTLE_ENDIAN(ip)
-#endif
-#endif
 
 bool g_b_exit = false;
 struct sigaction g_sigact;
@@ -191,6 +181,7 @@ void usage(const char *argv0)
            " log details level to <level>(0 <= level <= 3)\n");
     printf("  -s, --sockets=<list|range>\tLog only sockets that match <list> or <range>, format: "
            "4-16 or 1,9 (or combination)\n");
+    printf("  -C, --csv_file=<file path>\tA path to the statics CSV file\n");
     printf("  -V, --version\t\t\tPrint version\n");
     printf("  -h, --help\t\t\tPrint this help message\n");
 }
@@ -244,7 +235,6 @@ void update_delta_stat(socket_stats_t *p_curr_stat, socket_stats_t *p_prev_stat)
         (p_curr_stat->counters.n_rx_poll_hit - p_prev_stat->counters.n_rx_poll_hit) / delay;
     p_prev_stat->n_rx_ready_byte_count = p_curr_stat->n_rx_ready_byte_count;
     p_prev_stat->n_tx_ready_byte_count = p_curr_stat->n_tx_ready_byte_count;
-    p_prev_stat->n_rx_ready_byte_limit = p_curr_stat->n_rx_ready_byte_limit;
     p_prev_stat->counters.n_rx_ready_byte_max = p_curr_stat->counters.n_rx_ready_byte_max;
     p_prev_stat->counters.n_rx_ready_byte_drop =
         (p_curr_stat->counters.n_rx_ready_byte_drop - p_prev_stat->counters.n_rx_ready_byte_drop) /
@@ -370,6 +360,12 @@ void update_delta_ring_stat(ring_stats_t *p_curr_ring_stats, ring_stats_t *p_pre
             (p_curr_ring_stats->simple.n_tx_dropped_wqes -
              p_prev_ring_stats->simple.n_tx_dropped_wqes) /
             delay;
+        p_prev_ring_stats->simple.n_tx_num_bufs =
+            (p_curr_ring_stats->simple.n_tx_num_bufs - p_prev_ring_stats->simple.n_tx_num_bufs) /
+            delay;
+        p_prev_ring_stats->simple.n_zc_num_bufs =
+            (p_curr_ring_stats->simple.n_zc_num_bufs - p_prev_ring_stats->simple.n_zc_num_bufs) /
+            delay;
 #ifdef DEFINED_UTLS
         p_prev_ring_stats->n_tx_tls_contexts =
             (p_curr_ring_stats->n_tx_tls_contexts - p_prev_ring_stats->n_tx_tls_contexts) / delay;
@@ -385,6 +381,14 @@ void update_delta_ring_stat(ring_stats_t *p_curr_ring_stats, ring_stats_t *p_pre
             p_prev_ring_stats->tap.n_vf_plugouts =
                 (p_curr_ring_stats->tap.n_vf_plugouts - p_prev_ring_stats->tap.n_vf_plugouts);
         } else {
+            p_prev_ring_stats->simple.n_tx_tso_pkt_count =
+                (p_curr_ring_stats->simple.n_tx_tso_pkt_count -
+                 p_prev_ring_stats->simple.n_tx_tso_pkt_count) /
+                delay;
+            p_prev_ring_stats->simple.n_tx_tso_byte_count =
+                (p_curr_ring_stats->simple.n_tx_tso_byte_count -
+                 p_prev_ring_stats->simple.n_tx_tso_byte_count) /
+                delay;
             p_prev_ring_stats->simple.n_rx_interrupt_received =
                 (p_curr_ring_stats->simple.n_rx_interrupt_received -
                  p_prev_ring_stats->simple.n_rx_interrupt_received) /
@@ -471,6 +475,14 @@ void update_delta_global_stat(global_stats_t *p_curr_global_stats,
         p_prev_global_stats->n_pending_sockets =
             (p_curr_global_stats->n_pending_sockets - p_prev_global_stats->n_pending_sockets) /
             delay;
+        p_prev_global_stats->socket_tcp_destructor_counter =
+            (p_curr_global_stats->socket_tcp_destructor_counter.load() -
+             p_prev_global_stats->socket_tcp_destructor_counter.load()) /
+            delay;
+        p_prev_global_stats->socket_udp_destructor_counter =
+            (p_curr_global_stats->socket_udp_destructor_counter.load() -
+             p_prev_global_stats->socket_udp_destructor_counter.load()) /
+            delay;
     }
 }
 
@@ -484,6 +496,7 @@ void print_ring_stats(ring_instance_block_t *p_ring_inst_arr)
     }
 
     for (int i = 0; i < NUM_OF_SUPPORTED_RINGS; i++) {
+        // coverity[missing_lock:FALSE] /* Turn off coverity missing_lock check*/
         if (p_ring_inst_arr[i].b_enabled) {
             p_ring_stats = &p_ring_inst_arr[i].ring_stats;
             printf("======================================================\n");
@@ -532,6 +545,12 @@ void print_ring_stats(ring_instance_block_t *p_ring_inst_arr)
                 printf(FORMAT_STATS_32bit, "Tap fd:", p_ring_stats->tap.n_tap_fd);
                 printf(FORMAT_RING_TAP_NAME, "Tap Device:", p_ring_stats->tap.s_tap_name);
             } else {
+                if (p_ring_stats->simple.n_tx_tso_pkt_count ||
+                    p_ring_stats->simple.n_tx_tso_byte_count) {
+                    printf(FORMAT_RING_PACKETS, "TSO Offload:",
+                           p_ring_stats->simple.n_tx_tso_byte_count / BYTES_TRAFFIC_UNIT,
+                           p_ring_stats->simple.n_tx_tso_pkt_count, post_fix);
+                }
                 if (p_ring_stats->simple.n_rx_interrupt_requests ||
                     p_ring_stats->simple.n_rx_interrupt_received) {
                     printf(FORMAT_RING_INTERRUPT,
@@ -552,6 +571,11 @@ void print_ring_stats(ring_instance_block_t *p_ring_inst_arr)
                            p_ring_stats->simple.n_tx_dev_mem_pkt_count,
                            p_ring_stats->simple.n_tx_dev_mem_oob, post_fix);
                 }
+
+                printf(FORMAT_STATS_32bit,
+                       "TX buffers inflight:", p_ring_stats->simple.n_tx_num_bufs);
+                printf(FORMAT_STATS_32bit,
+                       "TX ZC buffers inflight:", p_ring_stats->simple.n_zc_num_bufs);
             }
         }
     }
@@ -568,6 +592,7 @@ void print_cq_stats(cq_instance_block_t *p_cq_inst_arr)
     }
 
     for (int i = 0; i < NUM_OF_SUPPORTED_CQS; i++) {
+        // coverity[missing_lock:FALSE] /* Turn off coverity missing_lock check*/
         if (p_cq_inst_arr[i].b_enabled) {
             p_cq_stats = &p_cq_inst_arr[i].cq_stats;
             printf("======================================================\n");
@@ -621,6 +646,7 @@ void print_bpool_stats(bpool_instance_block_t *p_bpool_inst_arr)
     }
 
     for (int i = 0; i < NUM_OF_SUPPORTED_BPOOLS; i++) {
+        // coverity[missing_lock:FALSE] /* Turn off coverity missing_lock check*/
         if (p_bpool_inst_arr && p_bpool_inst_arr[i].b_enabled) {
             p_bpool_stats = &p_bpool_inst_arr[i].bpool_stats;
             printf("======================================================\n");
@@ -632,6 +658,8 @@ void print_bpool_stats(bpool_instance_block_t *p_bpool_inst_arr)
                 printf("\tBUFFER_POOL=[%u]\n", i);
             }
             printf(FORMAT_STATS_32bit, "Size:", p_bpool_stats->n_buffer_pool_size);
+            printf(FORMAT_STATS_32bit, "Buffers in use:",
+                   p_bpool_stats->n_buffer_pool_created - p_bpool_stats->n_buffer_pool_size);
             printf(FORMAT_STATS_32bit, "No buffers error:", p_bpool_stats->n_buffer_pool_no_bufs);
             if (p_bpool_stats->n_buffer_pool_expands) {
                 printf(FORMAT_STATS_32bit, "Expands:", p_bpool_stats->n_buffer_pool_expands);
@@ -660,7 +688,11 @@ void print_global_stats(global_instance_block_t *p_global_inst_arr)
                    "No segments error:", p_global_stats->n_tcp_seg_pool_no_segs);
             printf("======================================================\n");
             printf("\tGLOBAL\n");
-            printf(FORMAT_STATS_32bit, "Pending sockets:", p_global_stats->n_pending_sockets);
+            printf(FORMAT_STATS_s_32bit, "Pending sockets:", p_global_stats->n_pending_sockets);
+            printf(FORMAT_STATS_s_32bit,
+                   "Destructed TCP sockets:", p_global_stats->socket_tcp_destructor_counter.load());
+            printf(FORMAT_STATS_s_32bit,
+                   "Destructed UDP sockets:", p_global_stats->socket_udp_destructor_counter.load());
         }
     }
     printf("======================================================\n");
@@ -1074,10 +1106,10 @@ void print_bpool_deltas(bpool_instance_block_t *p_curr_bpool_stats,
 void print_global_deltas(global_instance_block_t *p_curr_global_stats,
                          global_instance_block_t *p_prev_global_stats)
 {
-    if (unlikely(!p_curr_global_stats || !p_prev_global_stats)) {
-        return;
-    }
     for (int i = 0; i < NUM_OF_SUPPORTED_GLOBALS; i++) {
+        if (!p_curr_global_stats || !p_prev_global_stats) {
+            break;
+        }
         update_delta_global_stat(&p_curr_global_stats[i].global_stats,
                                  &p_prev_global_stats[i].global_stats);
     }
@@ -1180,6 +1212,7 @@ void add_fd_to_array(int fd, ip_addr mc_grp, mc_group_fds_t *mc_group_fds, int *
     }
     // the mc_group wasnt found
     // Add this mc group to the array
+    // coverity[copy_assignment_call] /* Turn off coverity warning for COPY_INSTEAD_OF_MOVE */
     mc_group_fds[i].mc_grp = mc_grp;
     int fd1 = fd;
     mc_group_fds[i].fd_list.push_back(fd1);
@@ -1194,7 +1227,6 @@ void print_mc_group_fds(mc_group_fds_t *mc_group_fds, int array_size)
     printf("------------------------------\n");
     for (int i = 0; i < array_size; i++) {
         char mcg_str[256];
-        /* cppcheck-suppress wrongPrintfScanfArgNum */
         sprintf(mcg_str, "[%s]", mc_group_fds[i].mc_grp.to_str().c_str());
         printf("%-22s", mcg_str);
         for (const auto &fd : mc_group_fds[i].fd_list) {
@@ -1221,10 +1253,9 @@ void show_mc_group_stats(mc_grp_info_t *p_mc_grp_info, socket_instance_block_t *
         size_t fd = (size_t)p_instance[i].skt_stats.fd;
         if (p_instance[i].b_enabled && g_fd_mask[fd]) {
             socket_stats_t *p_si_stats = &p_instance[i].skt_stats;
+            // coverity[missing_lock] /* Turn off coverity missing_lock check*/
             for (int grp_idx = 0; grp_idx < p_mc_grp_info->max_grp_num; grp_idx++) {
                 if (p_si_stats->mc_grp_map.test(grp_idx)) {
-                    // printf("fd %d Member of = [%d.%d.%d.%d]\n",p_si_stats->fd,
-                    // NIPQUAD(p_si_stats->mc_grp[grp_idx]));
                     add_fd_to_array(p_si_stats->fd, p_mc_grp_info->mc_grp_tbl[grp_idx].mc_grp,
                                     mc_group_fds, &array_size);
                 }
@@ -1397,6 +1428,11 @@ void stats_reader_handler(sh_mem_t *p_sh_mem, int pid)
     global_instance_block_t curr_global_blocks[NUM_OF_SUPPORTED_GLOBALS];
     iomux_stats_t prev_iomux_blocks;
     iomux_stats_t curr_iomux_blocks;
+    socket_listen_counter_aggregate socket_counters {user_params.print_details_mode == e_deltas};
+    tls_context_counters_show tls_counters {user_params.print_details_mode == e_deltas};
+    global_counters_show global_counters {user_params.print_details_mode == e_deltas};
+    cpu_usage_show cpu_usage;
+    ring_packet_aggregate ring_packets {user_params.print_details_mode == e_deltas};
 
     if (user_params.dump != DUMP_DISABLED) {
         static std::unordered_map<int, const char *> dump_type_names = {
@@ -1433,6 +1469,10 @@ void stats_reader_handler(sh_mem_t *p_sh_mem, int pid)
     memset((void *)curr_ring_blocks, 0, sizeof(ring_instance_block_t) * NUM_OF_SUPPORTED_RINGS);
     memset((void *)prev_bpool_blocks, 0, sizeof(bpool_instance_block_t) * NUM_OF_SUPPORTED_BPOOLS);
     memset((void *)curr_bpool_blocks, 0, sizeof(bpool_instance_block_t) * NUM_OF_SUPPORTED_BPOOLS);
+    for (int i = 0; i < NUM_OF_SUPPORTED_GLOBALS; i++) {
+        prev_global_blocks[i].init();
+        curr_global_blocks[i].init();
+    }
     memset((void *)prev_global_blocks, 0,
            sizeof(global_instance_block_t) * NUM_OF_SUPPORTED_GLOBALS);
     memset((void *)curr_global_blocks, 0,
@@ -1458,6 +1498,11 @@ void stats_reader_handler(sh_mem_t *p_sh_mem, int pid)
         }
     }
 
+    if (user_params.csv_stream.is_open()) {
+        user_params.csv_stream << "Date,Time," << ring_packets.hdr_val << socket_counters.hdr_val
+                               << tls_counters.hdr_val << global_counters.hdr_val
+                               << cpu_usage.hdr_val << "\n";
+    }
     set_signal_action();
 
     while (!g_b_exit && proc_running && cycles) {
@@ -1481,6 +1526,17 @@ void stats_reader_handler(sh_mem_t *p_sh_mem, int pid)
                    NUM_OF_SUPPORTED_GLOBALS * sizeof(global_instance_block_t));
             curr_iomux_blocks = p_sh_mem->iomux;
         }
+
+        if (user_params.csv_stream.is_open()) {
+            char buf[64] = "N/A,N/A,";
+            time_t t = time(nullptr);
+            strftime(buf, sizeof(buf), "%F,%T,", localtime(&t));
+            user_params.csv_stream
+                << buf << ring_packets.update(p_sh_mem) << socket_counters.update(p_sh_mem)
+                << tls_counters.update(p_sh_mem) << global_counters.update(p_sh_mem)
+                << cpu_usage.update() << "\n";
+        }
+
         switch (user_params.view_mode) {
         case e_full:
             ret = system("clear");
@@ -1761,12 +1817,16 @@ void zero_ring_stats(ring_stats_t *p_ring_stats)
     if (p_ring_stats->n_type == RING_TAP) {
         p_ring_stats->tap.n_vf_plugouts = 0;
     } else {
+        p_ring_stats->simple.n_tx_tso_pkt_count = 0;
+        p_ring_stats->simple.n_tx_tso_byte_count = 0;
         p_ring_stats->simple.n_rx_interrupt_received = 0;
         p_ring_stats->simple.n_rx_interrupt_requests = 0;
         p_ring_stats->simple.n_tx_dropped_wqes = 0;
         p_ring_stats->simple.n_tx_dev_mem_byte_count = 0;
         p_ring_stats->simple.n_tx_dev_mem_pkt_count = 0;
         p_ring_stats->simple.n_tx_dev_mem_oob = 0;
+        p_ring_stats->simple.n_tx_num_bufs = 0;
+        p_ring_stats->simple.n_zc_num_bufs = 0;
     }
 }
 
@@ -1879,15 +1939,27 @@ int main(int argc, char **argv)
         int c = 0;
         int option_index = 0;
 
-        static struct option long_options[] = {
-            {"interval", 1, NULL, 'i'},      {"cycles", 1, NULL, 'c'},  {"view", 1, NULL, 'v'},
-            {"details", 1, NULL, 'd'},       {"pid", 1, NULL, 'p'},     {"directory", 1, NULL, 'k'},
-            {"sockets", 1, NULL, 's'},       {"version", 0, NULL, 'V'}, {"zero", 0, NULL, 'z'},
-            {"log_level", 1, NULL, 'l'},     {"dump", 1, NULL, 0},      {"fd_dump", 1, NULL, 'S'},
-            {"details_level", 1, NULL, 'D'}, {"name", 1, NULL, 'n'},    {"find_pid", 0, NULL, 'f'},
-            {"forbid_clean", 0, NULL, 'F'},  {"help", 0, NULL, 'h'},    {0, 0, 0, 0}};
+        static struct option long_options[] = {{"interval", 1, NULL, 'i'},
+                                               {"cycles", 1, NULL, 'c'},
+                                               {"view", 1, NULL, 'v'},
+                                               {"details", 1, NULL, 'd'},
+                                               {"pid", 1, NULL, 'p'},
+                                               {"directory", 1, NULL, 'k'},
+                                               {"sockets", 1, NULL, 's'},
+                                               {"version", 0, NULL, 'V'},
+                                               {"zero", 0, NULL, 'z'},
+                                               {"log_level", 1, NULL, 'l'},
+                                               {"dump", 1, NULL, 0},
+                                               {"fd_dump", 1, NULL, 'S'},
+                                               {"details_level", 1, NULL, 'D'},
+                                               {"name", 1, NULL, 'n'},
+                                               {"find_pid", 0, NULL, 'f'},
+                                               {"forbid_clean", 0, NULL, 'F'},
+                                               {"help", 0, NULL, 'h'},
+                                               {"csv_file", 1, NULL, 'C'},
+                                               {0, 0, 0, 0}};
 
-        if ((c = getopt_long(argc, argv, "i:c:v:d:p:k:s:Vzl:S:D:n:fFh?", long_options,
+        if ((c = getopt_long(argc, argv, "i:c:v:d:p:k:s:Vzl:S:C:D:n:fFh?", long_options,
                              &option_index)) == -1) {
             break;
         }
@@ -2011,6 +2083,15 @@ int main(int argc, char **argv)
                 user_params.fd_dump_log_level = dump_log_level;
             }
         } break;
+        case 'C': {
+            user_params.csv_stream.open(optarg);
+            if (!user_params.csv_stream.is_open()) {
+                log_err("Unable to open file: %s", optarg);
+                usage(argv[0]);
+                cleanup(NULL);
+                return 1;
+            }
+        } break;
         case 'D': {
             errno = 0;
             int details_level = 0;
@@ -2246,3 +2327,45 @@ int print_processes_stats(const std::vector<int> &pids)
 
     return 0;
 }
+
+///////////////////////////
+void cpu_stats::capture()
+{
+    std::ifstream fileStat("/proc/stat");
+    std::string line;
+    std::getline(fileStat, line);
+    if (!line.compare(0, 3, "cpu")) {
+        read(line);
+    } else {
+        log_system_err("Read /proc/stat failed, unexpected line %s\n", line.c_str());
+        enabled = false;
+    }
+    fileStat.close();
+}
+
+void cpu_stats::read(const std::string &line)
+{
+    std::istringstream ss(line);
+    std::string label;
+    ss >> label;
+    for (int i = 0; i < TOTAL_CONTEXTS; ++i) {
+        ss >> times[i];
+    }
+}
+
+size_t cpu_stats::get_active_time() const
+{
+    return times[USER] + times[NICE] + times[SYSTEM] + times[IRQ] + times[SOFTIRQ] + times[STEAL] +
+        times[GUEST] + times[GUEST_NICE];
+}
+
+size_t cpu_stats::get_total_time() const
+{
+    return times[USER] + times[NICE] + times[SYSTEM] + times[IDLE] + times[IOWAIT] + times[IRQ] +
+        times[SOFTIRQ] + times[STEAL] + times[GUEST] + times[GUEST_NICE];
+}
+
+size_t cpu_stats::get_context_time(enum cpu_context context) const
+{
+    return times[context];
+}
