@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -421,7 +423,7 @@ TEST_F(xlio_send_zc, ti_2)
  *    Send data using few sendmsg(MSG_ZEROCOPY)
  * @details
  */
-TEST_F(xlio_send_zc, ti_3_few_send)
+TEST_F(xlio_send_zc, DISABLED_ti_3_few_send)
 {
     int rc = EOK;
     int test_iter = 3;
@@ -576,7 +578,7 @@ TEST_F(xlio_send_zc, ti_3_few_send)
  *    single call
  * @details
  */
-TEST_F(xlio_send_zc, ti_4_large_send)
+TEST_F(xlio_send_zc, DISABLED_ti_4_large_send)
 {
     int rc = EOK;
 
